/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var Randomizer = /** @class */ (function () {
    function Randomizer() {
    }
    Randomizer.prototype.create = function () {
        return null;
    };
    return Randomizer;
}());
var Integer = /** @class */ (function (_super) {
    __extends(Integer, _super);
    function Integer(min, max) {
        if (min === void 0) { min = 1; }
        if (max === void 0) { max = 9; }
        var _this = _super.call(this) || this;
        _this._min = min;
        _this._max = max;
        return _this;
    }
    Integer.prototype.create = function () {
        return Math.random() * (this._max - this._min + 1) + this._min | 0;
    };
    return Integer;
}(Randomizer));
var Double = /** @class */ (function (_super) {
    __extends(Double, _super);
    function Double(min, max) {
        if (min === void 0) { min = 0; }
        if (max === void 0) { max = 1; }
        var _this = _super.call(this) || this;
        _this._min = min;
        _this._max = max;
        return _this;
    }
    Double.prototype.create = function () {
        return Math.random() * (this._max - this._min) + this._min;
    };
    return Double;
}(Randomizer));
var String = /** @class */ (function (_super) {
    __extends(String, _super);
    function String(length, letters) {
        if (length === void 0) { length = 16; }
        if (letters === void 0) { letters = 'abcdefghijklmnopqrstuvwxyz'; }
        var _this = _super.call(this) || this;
        _this._length = length;
        _this._letters = letters;
        return _this;
    }
    String.prototype.create = function () {
        var text = '';
        var randomizer = new Integer(0, this._letters.length - 1);
        for (var i = 0; i < this._length; i++) {
            text += this._letters[randomizer.create()];
        }
        return text;
    };
    return String;
}(Randomizer));
var Array2D = /** @class */ (function (_super) {
    __extends(Array2D, _super);
    function Array2D(N, M, randomizer) {
        if (N === void 0) { N = 10; }
        if (M === void 0) { M = 10; }
        if (randomizer === void 0) { randomizer = new Integer(); }
        var _this = _super.call(this) || this;
        _this._N = N;
        _this._M = M;
        _this._randomizer = randomizer;
        _this._sorted = false;
        return _this;
    }
    Array2D.prototype.sorted = function (sorted) {
        if (sorted === void 0) { sorted = true; }
        this._sorted = sorted;
        return this;
    };
    Array2D.prototype.create = function () {
        var D = [];
        for (var i = 0; i < this._N; i++) {
            D.push([]);
            for (var j = 0; j < this._M; j++) {
                D[i].push(this._randomizer.create());
            }
            if (this._sorted)
                D[i].sort(function (a, b) { return a - b; });
        }
        return D;
    };
    return Array2D;
}(Randomizer));
var Array1D = /** @class */ (function (_super) {
    __extends(Array1D, _super);
    function Array1D(N, randomizer) {
        return _super.call(this, 1, N, randomizer) || this;
    }
    Array1D.prototype.create = function () {
        return _super.prototype.create.call(this)[0];
    };
    return Array1D;
}(Array2D));
var Graph = /** @class */ (function (_super) {
    __extends(Graph, _super);
    function Graph(N, ratio, randomizer) {
        if (N === void 0) { N = 5; }
        if (ratio === void 0) { ratio = .3; }
        if (randomizer === void 0) { randomizer = new Integer(); }
        var _this = _super.call(this) || this;
        _this._N = N;
        _this._ratio = ratio;
        _this._randomizer = randomizer;
        _this._directed = true;
        _this._weighted = false;
        return _this;
    }
    Graph.prototype.directed = function (directed) {
        if (directed === void 0) { directed = true; }
        this._directed = directed;
        return this;
    };
    Graph.prototype.weighted = function (weighted) {
        if (weighted === void 0) { weighted = true; }
        this._weighted = weighted;
        return this;
    };
    Graph.prototype.create = function () {
        var G = new Array(this._N);
        for (var i = 0; i < this._N; i++) {
            G[i] = new Array(this._N);
        }
        for (var i = 0; i < this._N; i++) {
            for (var j = 0; j < this._N; j++) {
                if (i === j) {
                    G[i][j] = 0;
                }
                else if (this._directed || i < j) {
                    G[i][j] = Math.random() < this._ratio ? this._weighted ? this._randomizer.create() : 1 : 0;
                }
                else {
                    G[i][j] = G[j][i];
                }
            }
        }
        return G;
    };
    return Graph;
}(Randomizer));
var Randomize = {
    Integer: Integer,
    Double: Double,
    String: String,
    Array1D: Array1D,
    Array2D: Array2D,
    Graph: Graph,
};

var MAX_COMMANDS = 1000000;
var MAX_OBJECTS = 100;
var Commander = /** @class */ (function () {
    function Commander(iArguments) {
        Commander.objectCount++;
        var className = this.constructor.name;
        this.key = Commander.keyRandomizer.create();
        this.command(className, iArguments);
    }
    Commander.command = function (key, method, iArguments) {
        var args = Array.from(iArguments);
        this.commands.push({
            key: key,
            method: method,
            args: JSON.parse(JSON.stringify(args)),
        });
        if (this.commands.length > MAX_COMMANDS)
            throw new Error('Too Many Commands');
        if (this.objectCount > MAX_OBJECTS)
            throw new Error('Too Many Objects');
    };
    Commander.prototype.destroy = function () {
        Commander.objectCount--;
        this.command('destroy', arguments);
    };
    Commander.prototype.command = function (method, iArguments) {
        Commander.command(this.key, method, iArguments);
    };
    Commander.prototype.toJSON = function () {
        return this.key;
    };
    Commander.keyRandomizer = new Randomize.String(8, 'abcdefghijklmnopqrstuvwxyz0123456789');
    Commander.objectCount = 0;
    Commander.commands = [];
    return Commander;
}());
var ALGORITHM_VISUALIZER = process.env.ALGORITHM_VISUALIZER;
if (!ALGORITHM_VISUALIZER) {
    var axios_1 = require('axios');
    var opn_1 = require('opn');
    process.on('beforeExit', function () {
        axios_1.post('https://algorithm-visualizer.org/api/visualizations', { content: JSON.stringify(Commander.commands) })
            .then(function (response) { return opn_1(response.data, { wait: false }); })
            .catch(console.error)
            .finally(function () { return process.exit(); });
    });
}

var Layout = /** @class */ (function (_super) {
    __extends(Layout, _super);
    function Layout(children) {
        return _super.call(this, arguments) || this;
    }
    Layout.setRoot = function (child) {
        this.command(null, 'setRoot', arguments);
    };
    Layout.prototype.add = function (child, index) {
        this.command('add', arguments);
    };
    Layout.prototype.remove = function (child) {
        this.command('remove', arguments);
    };
    Layout.prototype.removeAll = function () {
        this.command('removeAll', arguments);
    };
    return Layout;
}(Commander));

var VerticalLayout = /** @class */ (function (_super) {
    __extends(VerticalLayout, _super);
    function VerticalLayout() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return VerticalLayout;
}(Layout));

var HorizontalLayout = /** @class */ (function (_super) {
    __extends(HorizontalLayout, _super);
    function HorizontalLayout() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return HorizontalLayout;
}(Layout));

var Tracer = /** @class */ (function (_super) {
    __extends(Tracer, _super);
    function Tracer(title) {
        return _super.call(this, arguments) || this;
    }
    Tracer.delay = function (lineNumber) {
        this.command(null, 'delay', arguments);
    };
    Tracer.prototype.set = function () {
        this.command('set', arguments);
    };
    Tracer.prototype.reset = function () {
        this.command('reset', arguments);
    };
    return Tracer;
}(Commander));

var LogTracer = /** @class */ (function (_super) {
    __extends(LogTracer, _super);
    function LogTracer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LogTracer.prototype.set = function (log) {
        this.command('set', arguments);
    };
    LogTracer.prototype.print = function (message) {
        this.command('print', arguments);
    };
    LogTracer.prototype.println = function (message) {
        this.command('println', arguments);
    };
    LogTracer.prototype.printf = function (format) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this.command('printf', arguments);
    };
    return LogTracer;
}(Tracer));

var Array2DTracer = /** @class */ (function (_super) {
    __extends(Array2DTracer, _super);
    function Array2DTracer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Array2DTracer.prototype.set = function (array2d) {
        this.command('set', arguments);
    };
    Array2DTracer.prototype.patch = function (x, y, v) {
        this.command('patch', arguments);
    };
    Array2DTracer.prototype.depatch = function (x, y) {
        this.command('depatch', arguments);
    };
    Array2DTracer.prototype.select = function (sx, sy, ex, ey) {
        this.command('select', arguments);
    };
    Array2DTracer.prototype.selectRow = function (x, sy, ey) {
        this.command('selectRow', arguments);
    };
    Array2DTracer.prototype.selectCol = function (y, sx, ex) {
        this.command('selectCol', arguments);
    };
    Array2DTracer.prototype.deselect = function (sx, sy, ex, ey) {
        this.command('deselect', arguments);
    };
    Array2DTracer.prototype.deselectRow = function (x, sy, ey) {
        this.command('deselectRow', arguments);
    };
    Array2DTracer.prototype.deselectCol = function (y, sx, ex) {
        this.command('deselectCol', arguments);
    };
    return Array2DTracer;
}(Tracer));

var Array1DTracer = /** @class */ (function (_super) {
    __extends(Array1DTracer, _super);
    function Array1DTracer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Array1DTracer.prototype.set = function (array1d) {
        this.command('set', arguments);
    };
    Array1DTracer.prototype.patch = function (x, v) {
        this.command('patch', arguments);
    };
    Array1DTracer.prototype.depatch = function (x) {
        this.command('depatch', arguments);
    };
    Array1DTracer.prototype.select = function (sx, ex) {
        this.command('select', arguments);
    };
    Array1DTracer.prototype.deselect = function (sx, ex) {
        this.command('deselect', arguments);
    };
    Array1DTracer.prototype.chart = function (chartTracer) {
        this.command('chart', arguments);
    };
    return Array1DTracer;
}(Array2DTracer));

var ChartTracer = /** @class */ (function (_super) {
    __extends(ChartTracer, _super);
    function ChartTracer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ChartTracer;
}(Array1DTracer));

var GraphTracer = /** @class */ (function (_super) {
    __extends(GraphTracer, _super);
    function GraphTracer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GraphTracer.prototype.set = function (array2d) {
        this.command('set', arguments);
    };
    GraphTracer.prototype.directed = function (isDirected) {
        this.command('directed', arguments);
        return this;
    };
    GraphTracer.prototype.weighted = function (isWeighted) {
        this.command('weighted', arguments);
        return this;
    };
    GraphTracer.prototype.layoutCircle = function () {
        this.command('layoutCircle', arguments);
        return this;
    };
    GraphTracer.prototype.layoutTree = function (root, sorted) {
        this.command('layoutTree', arguments);
        return this;
    };
    GraphTracer.prototype.layoutRandom = function () {
        this.command('layoutRandom', arguments);
        return this;
    };
    GraphTracer.prototype.addNode = function (id, weight, x, y, visitedCount, selectedCount) {
        this.command('addNode', arguments);
    };
    GraphTracer.prototype.updateNode = function (id, weight, x, y, visitedCount, selectedCount) {
        this.command('updateNode', arguments);
    };
    GraphTracer.prototype.removeNode = function (id) {
        this.command('removeNode', arguments);
    };
    GraphTracer.prototype.addEdge = function (source, target, weight, visitedCount, selectedCount) {
        this.command('addEdge', arguments);
    };
    GraphTracer.prototype.updateEdge = function (source, target, weight, visitedCount, selectedCount) {
        this.command('updateEdge', arguments);
    };
    GraphTracer.prototype.removeEdge = function (source, target) {
        this.command('removeEdge', arguments);
    };
    GraphTracer.prototype.visit = function (target, source, weight) {
        this.command('visit', arguments);
    };
    GraphTracer.prototype.leave = function (target, source, weight) {
        this.command('leave', arguments);
    };
    GraphTracer.prototype.select = function (target, source) {
        this.command('select', arguments);
    };
    GraphTracer.prototype.deselect = function (target, source) {
        this.command('deselect', arguments);
    };
    GraphTracer.prototype.log = function (logTracer) {
        this.command('log', arguments);
    };
    return GraphTracer;
}(Tracer));

export { Array1DTracer, Array2DTracer, ChartTracer, Commander, GraphTracer, HorizontalLayout, Layout, LogTracer, Randomize, Tracer, VerticalLayout };
